package jace.config;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * A configurable field annotation means that an object property can be changed
 * by the end-user.
 * NOTE: Any field that implements this must be public and serializable!
 * If a field is not serializable, it will result in a serialization error
 * when the configuration is being saved.  There is no way to offer a compiler
 * warning to avoid this, unfortunately.
 * One way you can work with this constraint when allowing large reconfiguration
 * of functionality, such as Cards or other class implementations of hardware,
 * is to store the class itself of the component as a configuration value
 * and let the Reconfigure method generate a new instance.
 * @author brobert
 */
@Target(ElementType.FIELD)
@Retention(RetentionPolicy.RUNTIME)
public @interface ConfigurableField {
    public String name();
    public String shortName() default "";
    public String defaultValue() default "";
    public String description() default "";
    public String category() default "General";
}