package jace.config;

import jace.core.Utility;
import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 *
 * @author brobert
 */
public abstract class DynamicSelection<T> implements ISelection<T> {
    public DynamicSelection(T defaultValue) {
        setValue(defaultValue);
    }
    abstract public boolean allowNull();
    T currentValue;
    @Override
    public T getValue() {
        if (currentValue != null || !allowNull()) {
            return currentValue;
        } else {
            Iterator<? extends T> i = getSelections().keySet().iterator();
            return i.next();
        }
    }
    
    @Override
    public void setValue(T value) {currentValue = value;}
    
    public void setValueByMatch(String search) {
        Map<? extends T, String> selections = getSelections();
        String match = Utility.findBestMatch(search, selections.values());
        if (match != null) {
            for (T key : selections.keySet()) {
                if (selections.get(key).equals(match)) {
                    setValue(key);
                    return;
                }
            }
        } else {
            setValue(null);
        }
    }
}
