/*
 * CPU.java
 *
 * Created on January 4, 2007, 7:27 PM
 *
 */

package jace.core;

import jace.config.ConfigurableField;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author blurry
 */
public abstract class CPU extends Device {

    /** Creates a new instance of CPU */
    public CPU() {}

    @Override
    public String getShortName() {
        return "cpu";
    }
    private Debugger debugger = null;
    @ConfigurableField(name="Enable trace to STDOUT", shortName="trace")
    public boolean trace = false;
    public boolean isTraceEnabled() {return trace;}
    public void setTraceEnabled(boolean t) {trace = t;}
    @ConfigurableField(name = "Trace length", shortName="traceSize", description = "Number of most recent trace lines to keep for debugging errors.  Zero == disabled")
    public int traceLength = 0;
    private ArrayList<String> traceLog = new ArrayList<String>();
    public boolean isLogEnabled() {
        return (traceLength > 0);
    }
    public void log(String line) {
        if (!isLogEnabled()) return;
        while (traceLog.size() >= traceLength) {
            traceLog.remove(0);
        }
        traceLog.add(line);
    }
    public void dumpTrace() {
        System.out.println("Most recent " + traceLength + " instructions:");
        for (String s : traceLog) {
            System.out.println(s);
        }
        traceLog.clear();        
    }

    public void setDebug(Debugger d) {
        debugger = d;
        suspend();
    }
    
    public void clearDebug() {
        debugger = null;
        resume();
    }
    
    //@ConfigurableField(name="Program Counter")
    public int programCounter = 0;
    public int getProgramCounter() {
        return programCounter;
    }

    public void setProgramCounter(int programCounter) {
        this.programCounter = 0x00FFFF & programCounter;
    }
    
    public void incrementProgramCounter(int amount) {
        this.programCounter += amount;
        this.programCounter = 0x00FFFF & this.programCounter;
    }
        
    /**
     * Process a single tick of the main processor clock.
     * Either we're waiting to execute the next instruction, or the next instruction is ready to go
     */
    @Override
    public void tick() {
        if (debugger != null) {
            if (!debugger.isActive() && debugger.hasBreakpoints()) {
                for (int i : debugger.getBreakpoints()) {
                    if (i == getProgramCounter()) {
                        debugger.setActive(true);
                    }
                }
            }
            if (debugger.isActive()) {
                debugger.updateStatus();
                if (!debugger.takeStep()) {
                    // If the debugger is active and we aren't ready for the next step, sleep and exit
                    // Without the sleep, this would constitute a very rapid-fire loop and would eat
                    // an unnecessary amount of CPU.
                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException ex) {
                        Logger.getLogger(CPU.class.getName()).log(Level.SEVERE, null, ex);
                    }
                    return;
                }
            }
        }
        executeOpcode();
    }
    /*
     * Execute the current opcode at the current program counter
     *@return number of cycles to wait until next command can be executed
     */
    protected abstract void executeOpcode();
    public abstract void reset();
    public abstract void generateInterrupt();

    abstract public void pushPC();

    @Override
    public void attach() {

    }

    @Override
    public void detach() {
    }
}