/*
 * Card.java
 *
 * Created on February 1, 2007, 5:35 PM
 *
 */
package jace.core;

import jace.apple2e.SoftSwitches;

/**
 *
 * @author BLuRry
 */
public abstract class Card extends Device {

    private PagedMemory cxRom;
    private PagedMemory c8Rom;
    private int slot;
    private RAMListener ioListener;
    private RAMListener firmwareListener;
    private RAMListener c8firmwareListener;

    /**
     * Creates a new instance of Card
     */
    public Card() {
        cxRom = new PagedMemory(0x0100, PagedMemory.Type.cardFirmware);
        c8Rom = new PagedMemory(0x0800, PagedMemory.Type.cardFirmware);
    }

    @Override
    public String getShortName() {
        return "s" + getSlot();
    }

    @Override
    public String getName() {
        return getDeviceName() + " (slot "+slot+")";
    }

    abstract public void reset();

    @Override
    public void attach() {
        ioListener = new RAMListener(
                RAMEvent.TYPE.ANY,
                RAMEvent.SCOPE.RANGE,
                RAMEvent.VALUE.ANY) {
            protected void doConfig() {
                setScopeStart(slot * 16 + 0x00c080);
                setScopeEnd(slot * 16 + 0x00c08F);
            }

            protected void doEvent(RAMEvent e) {
                int address = e.getAddress() & 0x0f;
                handleIOAccess(address, e.getType(), e.getNewValue(), e);
            }
        };

        firmwareListener = new RAMListener(
                RAMEvent.TYPE.ANY,
                RAMEvent.SCOPE.RANGE,
                RAMEvent.VALUE.ANY) {
            protected void doConfig() {
                setScopeStart(slot * 256 + 0x00c000);
                setScopeEnd(slot * 256 + 0x00c0ff);
            }

            protected void doEvent(RAMEvent e) {
                Computer.getComputer().getMemory().setActiveCard(slot);
                if (SoftSwitches.CXROM.getState()) {
                    return;
                }
                handleFirmwareAccess(e.getAddress() & 0x0ff, e.getType(), e.getNewValue(), e);
            }
        };

        c8firmwareListener = new RAMListener(
                RAMEvent.TYPE.ANY,
                RAMEvent.SCOPE.RANGE,
                RAMEvent.VALUE.ANY) {
            protected void doConfig() {
                setScopeStart(slot * 256 + 0x00c800);
                setScopeEnd(slot * 256 + 0x00cfff);
            }

            protected void doEvent(RAMEvent e) {
                if (SoftSwitches.CXROM.getState()
                        || Computer.getComputer().getMemory().getActiveSlot() != getSlot()
                        || SoftSwitches.INTC8ROM.getState()) {
                    return;
                }
                handleC8FirmwareAccess(e.getAddress() - 0x0c800, e.getType(), e.getNewValue(), e);
            }
        };

        Computer.getComputer().getMemory().addListener(ioListener);
        Computer.getComputer().getMemory().addListener(firmwareListener);
        Computer.getComputer().getMemory().addListener(c8firmwareListener);
    }

    @Override
    public void detach() {
        suspend();
        Computer.getComputer().getMemory().removeListener(ioListener);
        Computer.getComputer().getMemory().removeListener(firmwareListener);
        Computer.getComputer().getMemory().removeListener(c8firmwareListener);
    }

    abstract protected void handleIOAccess(int register, RAMEvent.TYPE type, int value, RAMEvent e);

    abstract protected void handleFirmwareAccess(int register, RAMEvent.TYPE type, int value, RAMEvent e);

    abstract protected void handleC8FirmwareAccess(int register, RAMEvent.TYPE type, int value, RAMEvent e);

    public int getSlot() {
        return slot;
    }

    public void setSlot(int slot) {
        this.slot = slot;
    }

    public PagedMemory getCxRom() {
        return cxRom;
    }

    public PagedMemory getC8Rom() {
        return c8Rom;
    }

    @Override
    public void reconfigure() {
        boolean restart = suspend();
        detach();
        if (restart) {
            resume();
        }
        attach();
    }

    public void notifyVBLStateChanged(boolean state) {
        // Do nothing unless overridden
    }
    
    public boolean suspendWithCPU() {
        return false;
    }
}