package jace.core;

import jace.config.InvokableAction;
import jace.config.Reconfigurable;
import java.io.IOException;

/**
 *
 * @author blurry
 */
public abstract class Computer implements Reconfigurable {
    static private Computer theComputer;
    public RAM memory;
    public CPU cpu;
    public Video video;
    public Keyboard keyboard;
    
    /** Creates a new instance of Computer */
    public Computer() {
        theComputer = this;
        keyboard = new Keyboard();
    }

    public RAM getMemory() {
        return memory;
    }

    public void notifyVBLStateChanged(boolean state) {
        for (Card c : getMemory().cards) {
            if (c == null) continue;
            c.notifyVBLStateChanged(state);
        }
    }

    public void setMemory(RAM memory) {
        this.memory = memory;
    }
    
    public void waitForNextCycle() {
        //@TODO IMPLEMENT TIMER SLEEP CODE!
    }
    
    public Video getVideo() {
        return video;
    }

    public void setVideo(Video video) {
        this.video = video;
    }

    public CPU getCpu() {
        return cpu;
    }

    public void setCpu(CPU cpu) {
        this.cpu = cpu;
    }

    public void loadRom(String path) throws IOException {    
        memory.loadRom(path);
    }
    
    @InvokableAction(
            name="Cold boot",
            description="Process startup sequence from power-up",
            category="general",
            alternatives="Full reset;reset emulator"
    )
    public abstract void coldStart();
    @InvokableAction(
            name="Warm boot",
            description="Process user-initatiated reboot (ctrl+apple+reset)",
            category="general",
            alternatives="reboot;reset;three-finger-salute"
    )
    public abstract void warmStart();
    static public Computer getComputer() {
        return theComputer;
    }

    public Keyboard getKeyboard() {
        return this.keyboard;
    }
    
    protected abstract boolean isRunning();
    protected abstract void doPause();
    protected abstract void doResume();
    
    @InvokableAction(name = "Pause", description = "Stops the computer, allowing reconfiguration of core elements", alternatives = "freeze;halt")
    public static boolean pause() {
        boolean result = false;
        if (theComputer != null) {
            result = theComputer.isRunning();
            theComputer.doPause();
        }
        return result;
    }
    
    @InvokableAction(name = "Resume", description = "Resumes the computer if it was previously paused", alternatives = "unpause;unfreeze;resume")
    public static void resume() {
        if (theComputer != null) {
            theComputer.doResume();
        }
    }
}
