/*
 * Device.java
 *
 * Created on May 10, 2007, 5:46 PM
 */

package jace.core;

import jace.config.Reconfigurable;

/**
 *
 * @author brobert
 */
public abstract class Device implements Reconfigurable {
    // Number of cycles to do nothing (for cpu/video cycle accuracy)
    private int waitCycles=0;
    private boolean run = true;
    public boolean isPaused = false;

    public void addWaitCycles(int wait) {
        waitCycles += wait;
    }
    public void setWaitCycles(int wait) {
        waitCycles = wait;
    }
    
    public void doTick() {
        /*
        if (waitCycles <= 0)
            tick();
        else
            waitCycles--;
         */

        if (!run) {
//            System.out.println("Device stopped: " + getName());
            isPaused = true;
            return;
        }
        // The following might be as much as 7% faster than the above
        // My guess is that the above results in a GOTO
        // whereas the following pre-emptive return avoids that
        if (waitCycles > 0) {
            waitCycles--;
            return;
        }
        // Implicit else...
        tick();
    }

    public boolean isRunning() {
        return run;
    }

    public synchronized void setRun(boolean run) {
//        System.out.println(Thread.currentThread().getName() + (run ? " resuming " : " suspending ")+ getDeviceName());
        isPaused = false;
        this.run = run;
    }

    protected abstract String getDeviceName();

    @Override
    public String getName() {
        return getDeviceName();
    }

    public abstract void tick();
    public boolean suspend() {
        if (isRunning()) {
            setRun(false);
            return true;
        }
        return false;
    }
    public void resume() {setRun(true);}
    public abstract void attach();
    public abstract void detach();
}
