package jace.core;

import java.awt.event.KeyEvent;

/**
 * Listen for a specific key or set of keys
 * If there is a match, the handleKeyUp or handleKeyDown methods will be called.
 * This is meant to save a lot of extra conditional logic elsewhere.
 * 
 * The handler methods should return true if they have consumed the key event and do
 * not want any other processing to continue for that keypress.
 * @author brobert
 */
public abstract class KeyHandler {
    public int key = 0;
    public int modifiers = 0;
    public KeyHandler(int key, int... modifiers) {
        this.key = key;
        this.modifiers = 0;
        for (int m : modifiers) {
            this.modifiers |= m;
        }
    }
    
    public abstract boolean handleKeyUp(KeyEvent e);
    public abstract boolean handleKeyDown(KeyEvent e);
}
