/*
 * Motherboard.java
 *
 * Created on May 1, 2007, 11:22 PM
 */
package jace.core;

import jace.apple2e.SoftSwitches;
import jace.apple2e.Speaker;
import jace.config.ConfigurableField;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 *
 * @author Administrator
 */
public class Motherboard extends TimedDevice {

    static final Computer computer = Computer.getComputer();
    static final CPU cpu = computer.getCpu();
    static Motherboard instance;
    public static Card[] cards = computer.getMemory().getAllCards();
    static final public Set<Device> miscDevices = new HashSet<Device>();
    @ConfigurableField(name = "Enable Speaker", shortName = "speaker", defaultValue = "true")
    public static boolean enableSpeaker = true;
    private static Lock vblankLock = new ReentrantLock();
    private static Condition vblankStartCondition = vblankLock.newCondition();
    private static Condition vblankEndCondition = vblankLock.newCondition();
    public static Speaker speaker;
    public static SoundMixer mixer = new SoundMixer();

    static void vblankEnd() {
        try {
            vblankLock.lock();
            SoftSwitches.VBL.getSwitch().setState(true);
            computer.notifyVBLStateChanged(true);
            vblankEndCondition.signalAll();
        } catch (Exception e) {
        } finally {
            vblankLock.unlock();
        }
    }

    static void vblankStart() {
        try {
            vblankLock.lock();
            SoftSwitches.VBL.getSwitch().setState(false);
            computer.notifyVBLStateChanged(false);
            vblankStartCondition.signalAll();
        } catch (Exception e) {
        } finally {
            vblankLock.unlock();
        }
    }

    static public void waitForVBlank() {
        // Do nothing if it is already VBL
        if (!SoftSwitches.VBL.getState()) {
            return;
        }
        // Otherwise wait for VBL
        try {
            vblankLock.lock();
            while (SoftSwitches.VBL.getState()) {
                vblankStartCondition.await();
            }
        } catch (Exception e) {
        } finally {
            vblankLock.unlock();
        }
    }

    static public void waitForVBlankEnd() {
        // Do nothing if VBL is over already
        if (SoftSwitches.VBL.getState()) {
            return;
        }
        // Otherwise wait for VBL
        try {
            vblankLock.lock();
            while (!SoftSwitches.VBL.getState()) {
                vblankEndCondition.await();
            }
        } catch (Exception e) {
        } finally {
            vblankLock.unlock();
        }
    }

    static void waitForResync() throws InterruptedException {
        if (instance == null || instance.clock == null || !instance.clock.isAlive()) {
            Thread.sleep(10);
        } else {
            try {
                instance.clockLock.lock();
                instance.clockPulse.await();
            } catch (InterruptedException ex) {
                throw ex;
            } finally {
                instance.clockLock.unlock();
            }
        }

    }

    /**
     * Creates a new instance of Motherboard
     */
    public Motherboard() {
        instance = this;
    }

    protected String getDeviceName() {
        return "Motherboard";
    }

    @Override
    public String getShortName() {
        return "mb";
    }
    @ConfigurableField(category = "advanced", name = "CPU per clock", defaultValue = "1", description = "Number of CPU cycles per clock cycle (normal = 1)")
    public static int cpuPerClock = 1;
    public int clockCounter = 1;

    public void tick() {
        try {
            clockCounter--;
            cpu.doTick();
            if (clockCounter > 0) {
                return;
            }
            clockCounter = cpuPerClock;
            Computer.getComputer().getVideo().doTick();
            for (int i = 0; i < 7; i++) {
                if (cards[i] != null) {
                    cards[i].doTick();
                }
            }
            for (Device m : miscDevices) {
                if (m.isRunning()) {
                    m.doTick();
                }
            }
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }
    // From the holy word of Sather 3:5 (Table 3.1) :-)
    // This average speed averages in the "long" cycles
    public static long SPEED = 1020484L; // (NTSC)
    //public static long SPEED = 1015625L; // (PAL)

    public long defaultCyclesPerSecond() {
        return SPEED;
    }

    public synchronized void reconfigure() {
        boolean startAgain = suspend();
        accelorationRequestors.clear();
        super.reconfigure();
        cards = computer.getMemory().getAllCards();
        // Now create devices as needed, e.g. sound
        Motherboard.miscDevices.add(mixer);
        mixer.reconfigure();

        if (enableSpeaker) {
            try {
                if (speaker == null) {
                    speaker = new Speaker();
                } else {
                    speaker.attach();
                }
                if (mixer.lineAvailable) {
                    Motherboard.miscDevices.add(speaker);
                }
            } catch (Throwable t) {
                System.out.println("Unable to initalize sound -- deactivating speaker out");
                speaker.detach();
                Motherboard.miscDevices.remove(speaker);
            }
        } else {
            if (speaker != null) {
                speaker.detach();
                Motherboard.miscDevices.remove(speaker);
            }
        }
        if (startAgain) {
            resume();
        }
    }
    static HashSet<Object> accelorationRequestors = new HashSet<Object>();

    static public void requestSpeed(Object requester) {
        accelorationRequestors.add(requester);
        if (instance != null) {
            instance.enableTempMaxSpeed();
        }
    }

    static public void cancelSpeedRequest(Object requester) {
        accelorationRequestors.remove(requester);
        if (instance != null && accelorationRequestors.isEmpty()) {
            instance.disableTempMaxSpeed();
        }
    }

    @Override
    public void attach() {
    }
    Map<Card, Boolean> resume = new HashMap<Card, Boolean>();

    @Override
    public boolean suspend() {
        synchronized(resume) {
            resume.clear();
            for (Card c : cards) {
                if (c == null || !c.suspendWithCPU() || !c.isRunning()) {
                    continue;
                }
                resume.put(c, c.suspend());
            }
        }
        return super.suspend();
    }

    @Override
    public void resume() {
        super.resume();
        synchronized(resume) {
            for (Card c : cards) {
                if (Boolean.TRUE.equals(resume.get(c))) {
                    c.resume();     
                }
            }
        }
    }

    @Override
    public void detach() {
        for (Device d : miscDevices) {
            d.suspend();
        }
        miscDevices.clear();
        halt();
    }
}
