package jace.core;

import java.util.Arrays;

/**
 *
 * @author blurry
 */
public class PagedMemory {
    public enum Type {
        cardFirmware(0x0c800),
        languageCard(0x0d000),
        firmwareMain(0x0d000),
        firmware80column(0x0c300),
        slotRom(0x0c100),
        ram(0x0000);
        protected int baseAddress;
        private Type(int newBase) {
            baseAddress = newBase;
        }
        public int getBaseAddress() {return baseAddress;}
    }
    
    // This is a fixed array, used for internal-only!!
    private byte[][] internalMemory = new byte[0][];
    private Type type;
    
    /** Creates a new instance of PagedMemory */
    public PagedMemory(int size, Type memType) {
        type=memType;
        internalMemory = new byte[size>>8][256];
        for (int i=0; i<size; i+=256) {
            byte[] b = new byte[256];
            Arrays.fill(b, (byte) 0x00);
            internalMemory[i>>8] = b;
        }
    }
    
    public PagedMemory(byte[] romData, Type memType) {
        type=memType;
        loadData(romData);
    }

    public void loadData(byte[] romData) {
        for (int i=0; i<romData.length; i+=256) {
            byte[] b = new byte[256];
            for (int j=0; j < 256; j++)
                b[j]=romData[i+j];
            internalMemory[i>>8] = b;
        }
    }

    public void loadData(byte[] romData, int offset, int length) {
        for (int i=0; i<length; i+=256) {
            byte[] b = new byte[256];
            for (int j=0; j < 256; j++)
                b[j]=romData[offset+i+j];
            internalMemory[i>>8] = b;
        }
    }    
    
    public byte[][] getMemory() {return internalMemory;}
    public byte[] get(int pageNumber) {return internalMemory[pageNumber];}
    public void set(int pageNumber, byte[] bank) {
        internalMemory[pageNumber] = bank;
    }
    public byte[] getMemoryPage(int memoryBase) {
        int offset = memoryBase - type.baseAddress;
//        int page = offset >> 8;
        int page = (offset >> 8) & 0x0ff;
//        return get(page);
        return internalMemory[page];
    }
    public void setBanks(int sourceStart, int sourceLength, int targetStart, PagedMemory source) {
        for (int i=0; i < sourceLength; i++) {
            set(targetStart+i, source.get(sourceStart+i));
        }
    }

    public byte readByte(int address) {
        return getMemoryPage(address)[address%256];
    }
    
    public void writeByte(int address, byte value) {
        getMemoryPage(address)[address%256]=value;
    }    
    
    public void fillBanks(PagedMemory source) {
        byte[][] sourceMemory = source.getMemory();
        int sourceBase = source.type.getBaseAddress() / 256;
        int thisBase = type.getBaseAddress() / 256;
        int start = Math.max(sourceBase, thisBase);
        int sourceEnd = sourceBase + source.getMemory().length;
        int thisEnd = thisBase + getMemory().length;
        int end = Math.min(sourceEnd, thisEnd);
        for (int i=start; i < end; i++) {
            set(i - thisBase, sourceMemory[i - sourceBase]);
        }
    }
}