package jace.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 * @author blurry
 */
public abstract class SoftSwitch {

    private Boolean state;
    private Boolean initalState;
    private List<RAMListener> listeners;
    private final List<Integer> exclusionActivate = new ArrayList<Integer>();
    private final List<Integer> exclusionDeactivate = new ArrayList<Integer>();
    private final List<Integer> exclusionQuery = new ArrayList<Integer>();
    private String name;
    private boolean toggleType = false;

    /** Creates a new instance of SoftSwitch
     * @param name
     * @param initalState 
     */
    public SoftSwitch(String name, Boolean initalState) {
        this.initalState = initalState;
        this.state = initalState;
        this.listeners = new ArrayList<RAMListener>();
        this.name = name;
    }

    public SoftSwitch(String name, int offAddress, int onAddress, int queryAddress, RAMEvent.TYPE changeType, Boolean initalState) {
        if (onAddress == offAddress && onAddress != -1) {
            toggleType = true;
//            System.out.println("Switch " + name + " is a toggle type switch!");
        }
        this.initalState = initalState;
        this.state = initalState;
        this.listeners = new ArrayList<RAMListener>();
        this.name = name;
        int[] onAddresses = null;
        int[] offAddresses = null;
        int[] queryAddressList = null;
        if (onAddress >= 0) {
            onAddresses = new int[]{onAddress};
        }
        if (offAddress >= 0) {
            offAddresses = new int[]{offAddress};
        }
        if (queryAddress >= 0) {
            queryAddressList = new int[]{queryAddress};
        }
        init(offAddresses, onAddresses, queryAddressList, changeType);
    }

    public SoftSwitch(String name, int[] offAddrs, int[] onAddrs, int[] queryAddrs, RAMEvent.TYPE changeType, Boolean initalState) {
        this(name, initalState);
        init(offAddrs, onAddrs, queryAddrs, changeType);
    }

    private void init(int[] offAddrs, int[] onAddrs, int[] queryAddrs, RAMEvent.TYPE changeType) {
        if (toggleType) {
            List<Integer> addrs = new ArrayList<Integer>();
            for (int i : onAddrs) {
                addrs.add(i);
            }
            Collections.sort(addrs);
            final int beginAddr = addrs.get(0);
            final int endAddr = addrs.get(addrs.size() - 1);
            for (int i = beginAddr; i < endAddr; i++) {
                if (!addrs.contains(i)) {
                    exclusionActivate.add(i);
                }
            }
            RAMListener l = new RAMListener(changeType, RAMEvent.SCOPE.RANGE, RAMEvent.VALUE.ANY) {

                @Override
                protected void doConfig() {
                    setScopeStart(beginAddr);
                    setScopeEnd(endAddr);
                }

                @Override
                protected void doEvent(RAMEvent e) {
                    if (!exclusionActivate.contains(e.getAddress())) {
                        //                        System.out.println("Access to "+Integer.toHexString(e.getAddress())+" ENABLES switch "+getName());
                        setState(!getState());
                    }
                }
            };
            addListener(l);
        } else {
            if (onAddrs != null) {
                List<Integer> addrs = new ArrayList<Integer>();
                for (int i : onAddrs) {
                    addrs.add(i);
                }
                Collections.sort(addrs);
                final int beginAddr = addrs.get(0);
                final int endAddr = addrs.get(addrs.size() - 1);
                for (int i = beginAddr; i < endAddr; i++) {
                    if (!addrs.contains(i)) {
                        exclusionActivate.add(i);
                    }
                }
                RAMListener l = new RAMListener(changeType, RAMEvent.SCOPE.RANGE, RAMEvent.VALUE.ANY) {

                    @Override
                    protected void doConfig() {
                        setScopeStart(beginAddr);
                        setScopeEnd(endAddr);
                    }

                    @Override
                    protected void doEvent(RAMEvent e) {
                        if (!exclusionActivate.contains(e.getAddress())) {
  //                        System.out.println("Access to "+Integer.toHexString(e.getAddress())+" ENABLES switch "+getName());
                            setState(true);
                        }
                    }
                };
                addListener(l);
            }

            if (offAddrs != null) {
                List<Integer> addrs = new ArrayList<Integer>();
                for (int i : offAddrs) {
                    addrs.add(i);
                }
                final int beginAddr = addrs.get(0);
                final int endAddr = addrs.get(addrs.size() - 1);
                for (int i = beginAddr; i < endAddr; i++) {
                    if (!addrs.contains(i)) {
                        exclusionDeactivate.add(i);
                    }
                }
                RAMListener l = new RAMListener(changeType, RAMEvent.SCOPE.RANGE, RAMEvent.VALUE.ANY) {

                    @Override
                    protected void doConfig() {
                        setScopeStart(beginAddr);
                        setScopeEnd(endAddr);
                    }

                    @Override
                    protected void doEvent(RAMEvent e) {
                        if (!exclusionDeactivate.contains(e.getAddress())) {
                            setState(false);
//                          System.out.println("Access to "+Integer.toHexString(e.getAddress())+" disables switch "+getName());
                        }
                    }
                };
                addListener(l);
            }
        }

        if (queryAddrs != null) {
            List<Integer> addrs = new ArrayList<Integer>();
            for (int i : queryAddrs) {
                addrs.add(i);
            }
            final int beginAddr = addrs.get(0);
            final int endAddr = addrs.get(addrs.size() - 1);
            for (int i = beginAddr; i < endAddr; i++) {
                if (!addrs.contains(i)) {
                    exclusionQuery.add(i);
                }
            }
//            RAMListener l = new RAMListener(changeType, RAMEvent.SCOPE.RANGE, RAMEvent.VALUE.ANY) {
            RAMListener l = new RAMListener(RAMEvent.TYPE.READ, RAMEvent.SCOPE.RANGE, RAMEvent.VALUE.ANY) {

                @Override
                protected void doConfig() {
                    setScopeStart(beginAddr);
                    setScopeEnd(endAddr);
                }

                @Override
                protected void doEvent(RAMEvent e) {
                    if (!exclusionQuery.contains(e.getAddress())) {
                        e.setNewValue(0x0ff & readSwitch());
//                    System.out.println("Read from "+Integer.toHexString(e.getAddress())+" returns "+Integer.toHexString(e.getNewValue()));
                    }
                }
            };
            addListener(l);
        }

        register();
    }

    public boolean inhibit() {
        return false;
    }

    abstract protected byte readSwitch();

    protected void addListener(RAMListener l) {
        listeners.add(l);
    }

    public String getName() {
        return name;
    }

    public void reset() {
        if (initalState != null) {
            setState(initalState);
        }
    }

    public void register() {
        RAM m = Computer.getComputer().getMemory();
        for (RAMListener l : listeners) {
            m.addListener(l);
        }
    }

    public void unregister() {
        RAM m = Computer.getComputer().getMemory();
        for (RAMListener l : listeners) {
            m.removeListener(l);
        }
    }

    public void setState(boolean newState) {
        if (inhibit()) return;
//        if (this != SoftSwitches.VBL.getSwitch() &&
//            this != SoftSwitches.KEYBOARD.getSwitch())
//            System.out.println("Switch "+name+" set to "+newState);
        state = newState;
        /*
        if (queryAddresses != null) {
        RAM m = Computer.getComputer().getMemory();
        for (int i:queryAddresses) {
        byte old = m.read(i, false);
        m.write(i, (byte) (old & 0x7f | (state ? 0x080:0x000)), false);
        }
        }
         */
        stateChanged();
    }

    public final boolean getState() {
        if (state == null) return false;
        return state;
    }

    abstract public void stateChanged();

    @Override
    public String toString() {
        return getName()+(getState()?":1":":0");
    }
}