package jace.core;

import java.awt.image.BufferedImage;

/**
 *
 * @author blurry
 */
public abstract class VideoWriter {
    public abstract void displayByte(BufferedImage screen, int xOffset, int y, int yTextOffset, int yGraphicsOffset);

    // This is used to support composite mixed-mode writers so that we can talk to the writer being used for a scanline
    public VideoWriter actualWriter() {
        return this;
    }

    public abstract int getYOffset(int y);
    
    // Dirty flags allow us to know if a scanline has or has not changed
    // Very useful for knowing if we should bother drawing changes
    private boolean[] dirtyFlags = new boolean[192];
    public void markDirty(int y) {
        actualWriter().dirtyFlags[y]=true;
    }
    public void clearDirty(int y) {
        actualWriter().dirtyFlags[y]=false;        
    }
    public boolean isRowDirty(int y) {
        return actualWriter().dirtyFlags[y];
    }

    public boolean isMixed() {
        return false;
    }
}
