/*
 * CardDiskII.java
 *
 * Created on April 21, 2007
 *
 */
package jace.hardware;
// brendanr: Refactored as hardware

import jace.Emulator;
import jace.config.ConfigurableField;
import jace.config.Name;
import jace.config.Reconfigurable;
import jace.core.Card;
import jace.core.Motherboard;
import jace.core.RAMEvent;
import jace.core.RAMEvent.TYPE;
import jace.core.Utility;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;

@Name("Disk ][ Controller")
public class CardDiskII extends Card implements Reconfigurable {

    @ConfigurableField(name = "Disk 1", shortName="d1")
    public File disk1;
    @ConfigurableField(name = "Disk 2", shortName="d2")
    public File disk2;
    DiskIIDrive currentDrive;
    DiskIIDrive drive1;
    DiskIIDrive drive2;
    @ConfigurableField(category="Disk", defaultValue="254", name="Default volume", description="Value to use for disk volume number")
    static public int DEFAULT_VOLUME_NUMBER = 0x0FE;
    ImageIcon diskDrive1Icon;
    ImageIcon diskDrive2Icon;
    public CardDiskII() {
        try {
            loadRom("jace/data/DiskII.rom");
        } catch (IOException ex) {
            Logger.getLogger(CardDiskII.class.getName()).log(Level.SEVERE, null, ex);
        }
        diskDrive1Icon = Utility.loadIcon("disk_ii.png");
        diskDrive2Icon = Utility.loadIcon("disk_ii.png");
        drive1 = new DiskIIDrive();
        drive2 = new DiskIIDrive();
        reset();
    }

    @Override
    public String getDeviceName() {
        return "Disk ][ Controller";
    }

    @Override
    public void reset() {
        currentDrive = drive1;
        drive1.reset();
        drive2.reset();
        if (Emulator.getFrame() != null) {
            Emulator.getFrame().removeIndicators(this);
        }
//        Motherboard.cancelSpeedRequest(this);
    }

    @SuppressWarnings("fallthrough")
    @Override
    protected void handleIOAccess(int register, RAMEvent.TYPE type, int value, RAMEvent e) {
        // handle Disk ][ registers
        switch (register) {
            case 0x0:
            case 0x1:
            case 0x2:
            case 0x3:
            case 0x4:
            case 0x5:
            case 0x6:
            case 0x7:
                currentDrive.step(register);
                break;

            case 0x8:
                // drive off
                currentDrive.setOn(false);
//                Emulator.getFrame().removeIndicator(this, currentDrive == drive1 ? diskDrive1Icon : diskDrive2Icon, false);
                break;

            case 0x9:
                // drive on
                currentDrive.setOn(true);                
                Emulator.getFrame().addIndicator(this, currentDrive == drive1 ? diskDrive1Icon : diskDrive2Icon);
                break;

            case 0xA:
                // drive 1
                currentDrive = drive1;
                break;

            case 0xB:
                // drive 2
                currentDrive = drive2;
                break;

            case 0xC:
                // read/write latch
                currentDrive.write();
                e.setNewValue(currentDrive.readLatch());
                break;

            case 0xF:
                // write mode
                currentDrive.setWriteMode();
            case 0xD:
                // set latch
                if (e.getType() == RAMEvent.TYPE.WRITE) {
                    currentDrive.setLatchValue((byte) e.getNewValue());
                }
                e.setNewValue(currentDrive.readLatch());
                break;

            case 0xE:
                // read mode
                currentDrive.setReadMode();
                if (currentDrive.disk != null && currentDrive.disk.writeProtected)
                    e.setNewValue(0x080);
                else
//                    e.setNewValue((byte) (Math.random() * 256.0));
                    e.setNewValue(0);
                break;
        }
        // even addresses return the latch value
//        if (e.getType() == RAMEvent.TYPE.READ) {
//            if ((register & 0x1) == 0) {
//                e.setNewValue(currentDrive.latch);
//            } else {
//                // return floating bus value (IIRC)
//            }
//        }
        tweakTiming();
    }

    @Override
    protected void handleFirmwareAccess(int register, TYPE type, int value, RAMEvent e) {
        // Do nothing: The ROM does everything
        return;
    }

    public void loadRom(String path) throws IOException {
        InputStream romFile = CardDiskII.class.getClassLoader().getResourceAsStream(path);
        final int cxRomLength = 0x100;
        byte[] romData = new byte[cxRomLength];
        try {
            if (romFile.read(romData) != cxRomLength) {
                throw new IOException("Bad Disk ][ ROM size");
            }
            getCxRom().loadData(romData);
        } catch (IOException ex) {
            throw ex;
        }
    }

    @Override
    public void tick() {
        // Do nothing (if you want 1mhz timing control, you can do that here...)
//        drive1.tick();
//        drive2.tick();
    }

    @Override
    public void reconfigure() {
        diskDrive1Icon.setDescription("S"+getSlot()+"D1");
        diskDrive2Icon.setDescription("S"+getSlot()+"D2");
        try {
            if (disk1 == null) {
                drive1.ejectDisk();
            } else {
                drive1.insertDisk(disk1);
            }
            if (disk2 == null) {
                drive2.ejectDisk();
            } else {
                drive2.insertDisk(disk2);
            }
        } catch (IOException ex) {
            Logger.getLogger(CardDiskII.class.getName()).log(Level.SEVERE, null, ex);
        }
        super.reconfigure();
    }

    private void tweakTiming() {
        if (drive1.isOn() || drive2.isOn()) {
            Motherboard.requestSpeed(this);
        } else {
            Motherboard.cancelSpeedRequest(this);
        }
    }

    @Override
    protected void handleC8FirmwareAccess(int register, TYPE type, int value, RAMEvent e) {
        // There is no special c8 rom for this card
    }
}
