package jace.tracker;

import jace.hardware.CardMockingboard;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

/**
 *
 * @author blurry
 */
public class PlaybackEngine {
    static final int BITS = 16;
    static final int RATE = 44100;
    SourceDataLine sdl;
    AudioFormat af = new AudioFormat(RATE, BITS, 2, true, true);
    static final int BUFFER_SIZE = RATE/8;
    byte[] dataBuffer = new byte[BUFFER_SIZE * 4];
    int[] leftBuffer = new int[BUFFER_SIZE];
    int[] rightBuffer = new int[BUFFER_SIZE];
    CardMockingboard soundGen = null;
    
    /** Creates a new instance of PlaybackEngine */
    public PlaybackEngine(CardMockingboard gen) {
        soundGen = gen;
        try {
            init();
        } catch (LineUnavailableException ex) {
            ex.printStackTrace();
        }
    }
    
    private void init() throws LineUnavailableException {
        DataLine.Info dli = new DataLine.Info(SourceDataLine.class, af);
        if (AudioSystem.isLineSupported(dli)) {
            Line l = null;
            try {
                l = AudioSystem.getLine(dli);
            } catch (LineUnavailableException e) {
                throw e;
            }
            if (!(l instanceof SourceDataLine))
                throw new LineUnavailableException("Line is not an output line!");
            sdl = (SourceDataLine) l;
        } else
            throw new LineUnavailableException("Line not supported!");
        if (sdl == null)
            throw new LineUnavailableException("line not found");
    }

    LineListener listener = new LineListener() {
        public void update(LineEvent e) {
            play();
        }
    };

    Thread thread;
    
    boolean isPlaying = false;
    public void startPlayback() throws LineUnavailableException {
        if (!isPlaying) {
            isPlaying = true;
            sdl.addLineListener(listener);
            sdl.open(af);
            sdl.start();
        };
        if (thread == null || !thread.isAlive()) {
            thread = new Thread(){
                public void run() {
                    while (isPlaying) {
                        play();
                    }
                }
            };
            thread.setName("Playback engine");
            thread.start();
        }
    }
    
    public void stopPlayback() {
        isPlaying=false;
        sdl.stop();
        sdl.flush();
        sdl.removeLineListener(listener);
    }
    
    protected void finalize() throws Throwable {
        stopPlayback();
    }
    
    public void forcePlay() {
        if (sdl != null && sdl.isActive() && sdl.isOpen()) {
            sdl.flush();
            play();
        }
    }
    
    public void play() {
        int size = Math.min(sdl.available()/4, BUFFER_SIZE);
        while (size < BUFFER_SIZE / 4) {
            try {
                Thread.sleep(10);
            } catch (InterruptedException ex) {
                ex.printStackTrace();
            }
            size = Math.min(sdl.available()/4, BUFFER_SIZE);
        }
        if (soundGen != null) {
            soundGen.playSound(leftBuffer, rightBuffer);
        } else {
            System.out.println("NO GENERATOR IS PRESENT!");
        }
        for (int i=0; i < size; i++) {
            dataBuffer[i*4] = (byte) ((leftBuffer[i] & 0xff00) >>> 8);
            dataBuffer[i*4+1] = (byte) (leftBuffer[i] & 0xff);
            dataBuffer[i*4+2] = (byte) ((rightBuffer[i] & 0xff00) >>> 8);
            dataBuffer[i*4+3] = (byte) (rightBuffer[i] & 0xff);
        }
        
        sdl.write(dataBuffer, 0, size*4);
    }
}