/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * EmulatorFrame.java
 *
 * Created on Mar 27, 2010, 6:40:16 PM
 */
package jace.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.Shape;
import java.awt.event.KeyListener;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.JLabel;

/**
 *
 * @author brobert
 */
public class EmulatorFrame extends AbstractEmulatorFrame {

    @Override
    public String getShortName() {
        return "frame";
    }

    @Override
    public DebuggerPanel getDebuggerPanel() {
        return debuggerPanel;
    }

    @Override
    public Component getScreen() {
        return screen;
    }

    @Override
    public void resizeVideo() {
        super.resizeVideo();
        layers.setLayer(screen, layers.lowestLayer());
        layers.validate();
    }
    public static final long serialVersionUID = -1100;

    Font labelFont;
    
    /**
     * Creates new form EmulatorFrame
     */
    public EmulatorFrame() {
        initComponents();
        String[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        List<String> fontNames = Arrays.asList(fonts);
        if (fontNames.contains("Ubuntu")) {
            labelFont = Font.decode("Ubuntu-BOLD-14");
        } else {            
            labelFont = Font.decode("Arial-BOLD-14");
        }
        layers.setBounds(getContentPane().getBounds());
        screen.setBounds(layers.getBounds());
        debuggerPanel.setVisible(false);
        layers.setPosition(screen, 10);
        layers.setPosition(debuggerPanel, 2);
        reconfigure();
        screen.setFocusTraversalKeysEnabled(false);
        resizeVideo();
    }

    public synchronized void addKeyListener(KeyListener l) {
        super.addKeyListener(l);
        layers.addKeyListener(l);
        screen.addKeyListener(l);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        layers = new javax.swing.JLayeredPane();
        screen = new jace.ui.ScreenPanel();
        debuggerPanel = new jace.ui.DebuggerPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setBackground(new java.awt.Color(0, 0, 0));
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));

        layers.setBackground(new java.awt.Color(0, 0, 64));
        layers.setOpaque(true);

        javax.swing.GroupLayout screenLayout = new javax.swing.GroupLayout(screen);
        screen.setLayout(screenLayout);
        screenLayout.setHorizontalGroup(
            screenLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 560, Short.MAX_VALUE)
        );
        screenLayout.setVerticalGroup(
            screenLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 520, Short.MAX_VALUE)
        );

        screen.setBounds(0, 0, 560, 520);
        layers.add(screen, javax.swing.JLayeredPane.DEFAULT_LAYER);
        debuggerPanel.setBounds(460, 0, 100, 492);
        layers.add(debuggerPanel, javax.swing.JLayeredPane.MODAL_LAYER);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(layers, javax.swing.GroupLayout.DEFAULT_SIZE, 560, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(layers, javax.swing.GroupLayout.DEFAULT_SIZE, 518, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new EmulatorFrame().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public jace.ui.DebuggerPanel debuggerPanel;
    public javax.swing.JLayeredPane layers;
    public jace.ui.ScreenPanel screen;
    // End of variables declaration//GEN-END:variables
    Set<JLabel> previousIndicators = new HashSet<JLabel>();

    @Override
    public void doRedrawIndicators(Set<ImageIcon> ind) {
        if (previousIndicators != null) {
            for (JLabel l : previousIndicators) {
                l.setVisible(false);
                layers.remove(l);
            }
        }
        synchronized (previousIndicators) {
            previousIndicators.clear();
        }
        if (ind != null && !ind.isEmpty()) {            
            int x = layers.getWidth();
            int y = layers.getHeight();
            for (ImageIcon i : visibleIndicators) {
                JLabel label = createIndicatorIcon(i);
                x -= (i.getIconWidth() + 10);
                layers.add(label);
                layers.setLayer(label, layers.highestLayer());
                label.setBounds(x, y - i.getIconHeight() - 10, i.getIconWidth() + 5, i.getIconHeight() + 5);
                if (x <= 80) {
                    x = layers.getWidth();
                    y -= 140;
                }
                synchronized (previousIndicators) {
                    previousIndicators.add(label);
                }
                label.setVisible(true);
            }
        } else {
            screen.repaint();
        }
        screen.requestFocusInWindow();
    }

    @Override
    public void repaintIndicators() {
        synchronized (previousIndicators) {
            if (previousIndicators != null) {
                for (JLabel l : previousIndicators) {
                    Graphics g = l.getGraphics();
                    if (g != null) {
                        l.paint(g);
                    }
                }
            }
        }
    }

    Map<ImageIcon, JLabel> indicatorCache = new HashMap<ImageIcon, JLabel>(); 
    private JLabel createIndicatorIcon(ImageIcon i) {
        if (indicatorCache.containsKey(i)) {
            return indicatorCache.get(i);
        }
        JLabel label = new OutlinedLabel(i.getDescription());
        label.setIcon(i);
        label.setHorizontalTextPosition(JLabel.CENTER);
        label.setVerticalTextPosition(JLabel.CENTER);
        label.setBackground(Color.BLACK);
        label.setForeground(Color.WHITE);
        label.setFont(labelFont);
        label.setOpaque(false);
        label.setFocusable(false);
        label.setBounds(0,0,i.getIconWidth(), i.getIconHeight());
        BufferedImage img = new BufferedImage(i.getIconWidth(), i.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
        label.paint(img.getGraphics());
        ImageIcon icon = new ImageIcon(img);
        JLabel renderedLabel = new JLabel(icon);
        indicatorCache.put(i, renderedLabel);
        return renderedLabel;
    }
}
